from fipy import *

## NOTE: Distances are in micrometers
## Diffusion coefficient DP is in micrometer^2/s
## Times are in seconds

DP = 1.0e-6  # Diffusion coefficient
cP = 1.0e9   # Surface concentration of phosphorus
cB = 2.0e5   # Maximum displayed phosphorus concentration

LengthM = 19.0      # Length of device
ThickM  = 7.0       # thickness of wafer
gridThickM = 1.0e-1 # grid size

nx = int(LengthM/gridThickM)
ny = int(ThickM/gridThickM)

## To define opening on the surface where mask is applied for processing
mask = 5.0 # width of mask opening

FinalTime = 36000  # total amount of time simulated
dt = 100.0	   # Time step

mesh = Grid2D(dx=gridThickM, dy=gridThickM, nx=nx, ny=ny) + [[-LengthM/2.], [0.]]
var1 = CellVariable(mesh=mesh, value=0.0, hasOld=True)

viewer = Viewer(vars=var1, datamin=0.0, datamax=cB)
viewer.plot()
    
X, Y = mesh.getFaceCenters()

BCs= (FixedValue(faces=mesh.getFacesTop() & (-mask/2. <= X) & (X < mask/2.), value=cP))
    
## CORE OF THE CODE IS THE NEXT EQUATION
eq1=TransientTerm() == DiffusionTerm(DP)
    
t=0    
while t < FinalTime:
    t=t+dt
    var1.updateOld()
    eq1.solve(var1, dt=dt, boundaryConditions=BCs)
    if t % 1800 == 0:
	viewer.plot()
	print "t = ",t
	
